package renzhengshouquan

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/renzhengshouquan/model"
)

type profile struct {
	Auth
	order        string `data:"3"`
	modify       string `name:"modify info" role:"void"`
	upload       string `name:"upload" role:"void"`
	authList     string `name:"authList" role:"void"`
	fromList     string `name:"fromList" role:"void"`
	memberList   string `name:"memberList" role:"void"`
	serviceList  string `name:"serviceList" role:"void"`
	authPersonal string `name:"authPersonal" role:"void"`
}

func (s profile) Modify(m *ice.Message, arg ...string) {
	s.Update(m, kit.Dict(m.OptionSimple(model.INFO)), model.UID, m.Option(model.AUTH_UID))
}
func (s profile) Upload(m *ice.Message, arg ...string) {
	m.WarnNotValid(!kit.IsIn(m.OptionDefault(mdb.FIELD, aaa.AVATAR), aaa.AVATAR, aaa.BACKGROUND))
	s.Table.UploadUpdate(m, "", m.Option(model.AUTH_UID))
}
func (s profile) List(m *ice.Message, arg ...string) {
	if len(arg) == 1 {
		s.Select(m, model.UID, arg[0])
		kit.If(!m.IsTech() && s.IsVisitor(m), func() { m.Action() }, func() { m.Action(s.Modify) })
	}
	switch AuthType(kit.Int(m.Append(model.AUTH_TYPE))) {
	case AuthService:
		kit.If(!m.IsTech() && s.IsVisitor(m), func() { m.Action() }, func() { m.Action(s.Modify, s.Enter) })
		s.SelectJoinRecent(m, model.PLACE_UID)
		s.SelectJoinService(m)
		defer m.Echo(m.Append(model.CITY_NAME) + " " + m.Append(model.STREET_NAME) + " " + m.Append(model.PLACE_NAME))
	case AuthPersonal:
		s.SelectJoinUser(m)
		defer m.Echo(m.Append(model.NAME))
	default:
		defer m.Echo(m.Append(model.NAME))
	}
	s.RewriteAppend(m)
	m.EchoQRCode(s.Link(m, arg[0], m.PrefixKey()))
	m.Display("").DisplayCSS("")
}
func (s profile) FromList(m *ice.Message, arg ...string) {
	from := arg[0]
	for {
		msg := s.Select(m.Spawn(), model.UID, from)
		m.Copy(msg)
		if msg.Append(model.FROM_UID) != "" {
			from = msg.Append(model.FROM_UID)
		} else {
			break
		}
	}
}
func (s profile) AuthList(m *ice.Message, arg ...string) {
	m.Cmdy(s.Auth, m.Option(model.AUTH_UID)).Table(func(value ice.Maps) {
		if value[model.PLACE_UID] == "" {
			m.PushButton()
		} else {
			m.PushButton(s.Enter)
		}
	})
	s.SelectJoinService(m)
	s.RewriteAppend(m)
}
func (s profile) MemberList(m *ice.Message, arg ...string) {
	msg := s.Select(m.Spawn(), model.UID, m.Option(model.AUTH_UID))
	service := s.Table.ServiceList(m, msg.Append(model.PLACE_UID), msg.Append(model.SERVICE_UID))
	m.Cmdy(web.SPACE, service.Append(model.SPACE), service.Append(model.INDEX), ctx.ACTION, m.ActionKey(), msg.Append(model.PLACE_UID))
	m.Table(func(value ice.Maps) {
		if value[model.AUTH_STATUS] == "" && value[model.USER_UID] == m.Option(model.USER_UID) {
			m.PushButton(s.AuthPersonal)
		} else {
			m.PushButton()
		}
	})
	s.RewriteAppend(m)
}
func (s profile) ServiceList(m *ice.Message, arg ...string) {
	list := s.SettingProfile(m.Spawn(), arg...).Appendv(model.PLACE_UID)
	s.FieldsWithCreatedAT(m, s.Auth, kit.TransArgs(kit.Split(m.Config(mdb.FIELDS), ",", ","))...)
	s.SelectList(m, kit.Simple(model.PLACE_UID, kit.Sort(list))...)
	s.SelectJoinService(m).PushAction(s.Enter)
	s.RewriteAppend(m)
}
func (s profile) Enter(m *ice.Message, arg ...string) {}
func (s profile) AuthPersonal(m *ice.Message, arg ...string) {
	m.Cmdy(m.Prefix("credit"), m.ActionKey())
}

func init() { ice.TeamCtxCmd(profile{}) }
